@@ -22,87 +22,127 @@ class ForwardProblem;
 
 class BackwardProblem {
   public:
+    /**
+     * @brief Construct backward problem from forward problem
+     * @param fwd pointer to corresponding forward problem
+     * @return new BackwardProblem instance
+     */
+    explicit BackwardProblem(const ForwardProblem *fwd);
+
+    /**
+     * @brief Solve the backward problem.
+     *
+     * If adjoint sensitivities are enabled this will also compute
+     * sensitivities. workForwardProblem must be called before this is
+     * function is called.
+     */
     void workBackwardProblem();
 
-    BackwardProblem(ForwardProblem *fwd);
-    
-    /** accessor for t
+    /**
+     * @brief Accessor for current time t
      * @return t
      */
     realtype gett() const {
         return t;
     }
-    
-    /** accessor for which
+
+    /**
+     * @brief Accessor for which
      * @return which
      */
     int getwhich() const {
         return which;
     }
-    
-    /** accessor for pointer to which
+
+    /**
+     * @brief Accessor for pointer to which
      * @return which
      */
     int *getwhichptr() {
         return &which;
     }
-    
+
     /** accessor for pointer to xB
      * @return &xB
      */
     AmiVector *getxBptr() {
         return &xB;
     }
-    
-    /** accessor for pointer to xQB
+
+    /**
+     * @brief accessor for pointer to xQB
      * @return &xQB
      */
     AmiVector *getxQBptr() {
         return &xQB;
     }
-    
+
     /** accessor for pointer to dxB
      * @return &dxB
      */
     AmiVector *getdxBptr() {
         return &dxB;
     }
-    
-    /** accessor for dJydx
+
+    /**
+     * @brief Accessor for dJydx
      * @return dJydx
      */
-    std::vector<realtype> getdJydx() const {
+    std::vector<realtype> const& getdJydx() const {
         return dJydx;
     }
 
   private:
-    
+    /**
+     * @brief Execute everything necessary for the handling of events
+     * for the backward problem
+     *
+     * @param iroot index of event @type int
+     */
+    void handleEventB(int iroot);
+
+    /**
+     * @brief Execute everything necessary for the handling of data
+     * points for the backward problems
+     *
+     * @param it index of data point @type int
+     */
+    void handleDataPointB(const int it);
+
+
+    /**
+     * @brief Compute the next timepoint to integrate to.
+     *
+     * This is the maximum of tdata and troot but also takes into account if
+     * it<0 or iroot<0 where these expressions do not necessarily make sense.
+     *
+     * @param troot timepoint of next event @type realtype
+     * @param iroot index of next event @type int
+     * @param it index of next data point @type int
+     * @param model pointer to model specification object @type Model
+     * @return tnext next timepoint @type realtype
+     */
+    realtype getTnext(std::vector<realtype> const& troot, const int iroot, const int it);
+
+    /**
+     * @brief Compute likelihood sensitivities.
+     */
+    void computeLikelihoodSensitivities();
+
     Model *model;
     ReturnData *rdata;
     Solver *solver;
-    
-    void handleEventB(int iroot);
-    
-    void handleDataPointB(int it);
-    
-    void updateHeavisideB(int iroot);
-    
-    realtype getTnext(const realtype *troot, const int iroot, const int it);
-    
+
     /** current time */
     realtype t;
     /** parameter derivative of likelihood array */
     std::vector<realtype> llhS0;
     /** adjoint state vector */
     AmiVector xB;
-    /** old adjoint state vector */
-    AmiVector xB_old;
     /** differential adjoint state vector */
     AmiVector dxB;
     /** quadrature state vector */
     AmiVector xQB;
-    /** old quadrature state vector */
-    AmiVector xQB_old;
     /** array of state vectors at discontinuities*/
     const AmiVectorArray x_disc;
     /** array of differential state vectors at discontinuities*/
@@ -124,7 +164,7 @@ class BackwardProblem {
     int iroot = 0;
     /** array of index which root has been found */
     const std::vector<int> rootidx;
-    
+
     /** state derivative of data likelihood */
     const std::vector<realtype> dJydx;
     /** state derivative of event likelihood */
