@@ -42,7 +42,21 @@ void serialize(Archive &ar, amici::Solver &u, const unsigned int version) {
     ar &u.sensi;
     ar &u.atol;
     ar &u.rtol;
+    ar &u.atolB;
+    ar &u.rtolB;
+    ar &u.atol_fsa;
+    ar &u.rtol_fsa;
+    ar &u.quad_atol;
+    ar &u.quad_rtol;
+    ar &u.ss_atol;
+    ar &u.ss_rtol;
+    ar &u.ss_atol_sensi;
+    ar &u.ss_rtol_sensi;
     ar &u.maxsteps;
+    ar &u.maxstepsB;
+    ar &u.newton_preeq;
+    ar &u.newton_maxsteps;
+    ar &u.newton_maxlinsteps;
     ar &u.ism;
     ar &u.sensi_meth;
     ar &u.linsol;
@@ -61,8 +75,10 @@ void serialize(Archive &ar, amici::CVodeSolver &u, const unsigned int version) {
 
 template <class Archive>
 void serialize(Archive &ar, amici::Model &u, const unsigned int version) {
-    ar &const_cast<int &>(u.nx);
-    ar &const_cast<int &>(u.nxtrue);
+    ar &const_cast<int &>(u.nx_rdata);
+    ar &const_cast<int &>(u.nxtrue_rdata);
+    ar &const_cast<int &>(u.nx_solver);
+    ar &const_cast<int &>(u.nxtrue_solver);
     ar &const_cast<int &>(u.ny);
     ar &const_cast<int &>(u.nytrue);
     ar &const_cast<int &>(u.nz);
@@ -71,11 +87,12 @@ void serialize(Archive &ar, amici::Model &u, const unsigned int version) {
     ar &const_cast<int &>(u.nw);
     ar &const_cast<int &>(u.ndwdx);
     ar &const_cast<int &>(u.ndwdp);
+    ar &const_cast<int &>(u.ndxdotdw);
     ar &const_cast<int &>(u.nnz);
     ar &const_cast<int &>(u.nJ);
     ar &const_cast<int &>(u.ubw);
     ar &const_cast<int &>(u.lbw);
-    ar &const_cast<amici::AMICI_o2mode &>(u.o2mode);
+    ar &const_cast<amici::SecondOrderMode &>(u.o2mode);
     ar &const_cast<std::vector<int> &>(u.z2event);
     ar &const_cast<std::vector<realtype> &>(u.idlist);
     ar &u.h;
@@ -89,7 +106,7 @@ void serialize(Archive &ar, amici::Model &u, const unsigned int version) {
     ar &u.nmaxevent;
     ar &u.pscale;
     ar &u.tstart;
-
+    ar &u.stateIsNonNegative;
 }
 
 
@@ -99,6 +116,7 @@ void serialize(Archive &ar, amici::ReturnData &r, const unsigned int version) {
     ar &const_cast<int &>(r.np);
     ar &const_cast<int &>(r.nk);
     ar &const_cast<int &>(r.nx);
+    ar &const_cast<int &>(r.nx_solver);
     ar &const_cast<int &>(r.nxtrue);
     ar &const_cast<int &>(r.ny);
     ar &const_cast<int &>(r.nytrue);
@@ -111,11 +129,11 @@ void serialize(Archive &ar, amici::ReturnData &r, const unsigned int version) {
     ar &const_cast<int &>(r.nt);
     ar &const_cast<int &>(r.newton_maxsteps);
     ar &r.pscale;
-    ar &const_cast<amici::AMICI_o2mode &>(r.o2mode);
-    ar &const_cast<amici::AMICI_sensi_order &>(r.sensi);
-    ar &const_cast<amici::AMICI_sensi_meth &>(r.sensi_meth);
+    ar &const_cast<amici::SecondOrderMode &>(r.o2mode);
+    ar &const_cast<amici::SensitivityOrder &>(r.sensi);
+    ar &const_cast<amici::SensitivityMethod &>(r.sensi_meth);
 
-    ar &r.ts;
+    ar &const_cast<std::vector<realtype> &>(r.ts);
     ar &r.xdot;
     ar &r.J;
     ar &r.z & r.sigmaz;
@@ -139,9 +157,12 @@ void serialize(Archive &ar, amici::ReturnData &r, const unsigned int version) {
     ar &r.order;
 
     ar &r.newton_status;
-    ar &r.newton_time;
+    ar &r.newton_cpu_time;
     ar &r.newton_numsteps;
     ar &r.newton_numlinsteps;
+    ar &r.wrms_steadystate;
+    ar &r.wrms_sensi_steadystate;
+    ar &r.t_steadystate;
     ar &r.x0;
     ar &r.sx0;
     ar &r.llh;
@@ -164,24 +185,28 @@ char *serializeToChar(T const& data, int *size) {
      *
      * @param data input object
      * @param size maximum char length
-
+     *
      * @return The object serialized as char
      */
-    std::string serialized;
-    ::boost::iostreams::back_insert_device<std::string> inserter(serialized);
-    ::boost::iostreams::stream<::boost::iostreams::back_insert_device<std::string>>
-        s(inserter);
-    ::boost::archive::binary_oarchive oar(s);
-    oar << data;
-    s.flush();
+    try {
+        std::string serialized;
+        ::boost::iostreams::back_insert_device<std::string> inserter(serialized);
+        ::boost::iostreams::stream<::boost::iostreams::back_insert_device<std::string>>
+            s(inserter);
+        ::boost::archive::binary_oarchive oar(s);
+        oar << data;
+        s.flush();
 
-    char *charBuffer = new char[serialized.size()];
-    memcpy(charBuffer, serialized.data(), serialized.size());
+        char *charBuffer = new char[serialized.size()];
+        memcpy(charBuffer, serialized.data(), serialized.size());
 
-    if (size)
-        *size = serialized.size();
+        if (size)
+            *size = serialized.size();
 
-    return charBuffer;
+        return charBuffer;
+    } catch(boost::archive::archive_exception const& e) {
+        throw AmiException("Serialization to char failed: %s", e.what());
+    }
 }
 
 
@@ -196,14 +221,18 @@ T deserializeFromChar(const char *buffer, int size) {
      *
      * @return The deserialized object
      */
-    ::boost::iostreams::basic_array_source<char> device(buffer, size);
-    ::boost::iostreams::stream<::boost::iostreams::basic_array_source<char>> s(
-        device);
-    ::boost::archive::binary_iarchive iar(s);
-    T data;
-    iar >> data;
+    try {
+        ::boost::iostreams::basic_array_source<char> device(buffer, size);
+        ::boost::iostreams::stream<::boost::iostreams::basic_array_source<char>> s(
+            device);
+        ::boost::archive::binary_iarchive iar(s);
+        T data;
+        iar >> data;
 
-    return data;
+        return data;
+    } catch(::boost::archive::archive_exception const& e) {
+        throw AmiException("Deserialization from char failed: %s", e.what());
+    }
 }
 
 
@@ -213,19 +242,24 @@ std::string serializeToString(T const& data) {
      * @brief Serialize object to string
      *
      * @param data input object
-
+     *
      * @return The object serialized as string
      */
-    std::string serialized;
-    ::boost::iostreams::back_insert_device<std::string> inserter(serialized);
-    ::boost::iostreams::stream<
-        ::boost::iostreams::back_insert_device<std::string>>
-        s(inserter);
-    ::boost::archive::binary_oarchive oar(s);
-    oar << data;
-    s.flush();
+    try {
+        std::string serialized;
+        ::boost::iostreams::back_insert_device<std::string> inserter(serialized);
+        ::boost::iostreams::stream<
+            ::boost::iostreams::back_insert_device<std::string>>
+            s(inserter);
+        ::boost::archive::binary_oarchive oar(s);
+    
+        oar << data;
+        s.flush();
 
-    return serialized;
+        return serialized;
+    } catch(::boost::archive::archive_exception const& e) {
+        throw AmiException("Serialization to string failed: %s", e.what());
+    }
 }
 
 template <typename T>
@@ -234,20 +268,25 @@ std::vector<char> serializeToStdVec(T const& data) {
      * @brief Serialize object to std::vector<char>
      *
      * @param data input object
-
+     *
      * @return The object serialized as std::vector<char>
      */
-    std::string serialized;
-    ::boost::iostreams::back_insert_device<std::string> inserter(serialized);
-    ::boost::iostreams::stream<::boost::iostreams::back_insert_device<std::string>>
-        s(inserter);
-    ::boost::archive::binary_oarchive oar(s);
-    oar << data;
-    s.flush();
+    try{
+        std::string serialized;
+        ::boost::iostreams::back_insert_device<std::string> inserter(serialized);
+        ::boost::iostreams::stream<::boost::iostreams::back_insert_device<std::string>>
+            s(inserter);
+        ::boost::archive::binary_oarchive oar(s);
 
-    std::vector<char> buf(serialized.begin(), serialized.end());
-
-    return buf;
+        oar << data;
+        s.flush();
+        
+        std::vector<char> buf(serialized.begin(), serialized.end());
+        
+        return buf;
+    } catch(::boost::archive::archive_exception const& e) {
+        throw AmiException("Serialization to StdVec failed: %s", e.what());
+    }
 }
 
 template <typename T>
@@ -259,15 +298,20 @@ T deserializeFromString(std::string const& serialized) {
      *
      * @return The deserialized object
      */
-    ::boost::iostreams::basic_array_source<char> device(serialized.data(),
-                                                      serialized.size());
-    ::boost::iostreams::stream<::boost::iostreams::basic_array_source<char>> s(
-        device);
-    ::boost::archive::binary_iarchive iar(s);
-    T deserialized;
-    iar >> deserialized;
+    try{
+        ::boost::iostreams::basic_array_source<char> device(serialized.data(),
+                                                          serialized.size());
+        ::boost::iostreams::stream<::boost::iostreams::basic_array_source<char>> s(
+            device);
+        ::boost::archive::binary_iarchive iar(s);
+        T deserialized;
+    
+        iar >> deserialized;
 
-    return deserialized;
+        return deserialized;
+    } catch(::boost::archive::archive_exception const& e) {
+        throw AmiException("Deserialization from StdVec failed: %s", e.what());
+    }
 }
 
 
